/************************************************************************/
/*                                                                      */
/*      Print Edit  -  Firefox Extension  -  Main Window                */
/*                                                                      */
/*      Javascript for Main Window overlay                              */
/*                                                                      */
/*      Copyright (C) 2010  by  David Wilson                            */
/*                                                                      */
/*      Last Edit  -  17 Sep 2010                                       */
/*                                                                      */
/************************************************************************/

/* Refer to mozilla-central/toolkit/components/printing/content/printUtils.js */
/* Refer to mozilla-central/layout/base/nsDocumentViewer.cpp */

var printEdit =
{
    /********************************************************************/

    /* Shared variables */

    appInfo: Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo),
    versionComparator: Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator),
    ffVersion: "",

    prefs: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("printedit."),

    observerService: Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService),

    sourceBrowser: null,
    previewBrowser: null,
    
    menuPrintEdit: false,

    curMode: 0,

    cmdNumber: 0,

    highlightElement: null,
    highlightBox: null,

    selectCount: new Array(),
    selectElement: new Array(),  /* two dimensional */
    selectBox: new Array(),

    debugCount: 0,

    /********************************************************************/

    /* Box object constructor */

    Box: function()
    {
        this.container = null;
        this.sides = new Array();
    },

    /********************************************************************/

    /* Listen for changes to settings */

    prefsObserver:
    {
        register: function()
        {
            /* Query nsIPrefBranch2 interface to observe changes to pref values */
            printEdit.prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
            
            /* Add the observer */
            printEdit.prefs.addObserver("",this,false);
        },
        
        observe: function(subject,topic,data)
        {
            var i,menu,keyset;
            
            if (topic != "nsPref:changed") return;
            
            /* Re-apply print stylesheets */
            
            printEdit.displayCSSMediaPrint(printEdit.prefs.getIntPref("display-mediaprint"));
            
            /* Re-draw selection boxes */
            
            for (selectIndex = 0; selectIndex < printEdit.selectCount[printEdit.cmdNumber]; selectIndex++)
            {
                printEdit.showBox(printEdit.selectBox[selectIndex],printEdit.selectElement[printEdit.cmdNumber][selectIndex]);
            }
        }
    },

    /********************************************************************/

    /* Listen for HTTP requests to block loads */

    httpObserver:
    {
        register: function()
        {
            printEdit.observerService.addObserver(printEdit.httpObserver,"http-on-modify-request",false);
        },
        
        unregister: function()
        {
            printEdit.observerService.removeObserver(printEdit.httpObserver,"http-on-modify-request");
        },
        
        observe: function(subject,topic,data)
        {
            var httpChannel,notificationCallbacks,domWindow,browser;
            
            if (topic != "http-on-modify-request") return;
            
            httpChannel = subject.QueryInterface(Components.interfaces.nsIHttpChannel);
            
            notificationCallbacks = httpChannel.notificationCallbacks;
            if (!notificationCallbacks) notificationCallbacks = httpChannel.loadGroup.notificationCallbacks;
            
            if (notificationCallbacks)
            {
                try
                {
                    domWindow = notificationCallbacks.getInterface(Components.interfaces.nsIDOMWindow);
                }
                catch (e)
                {
                    return;  /* HTTP request has no associated window */
                }
                
                browser = gBrowser.getBrowserForDocument(domWindow.top.document);  /* can be null for RSS feed updates, etc */
                
                if (browser == printEdit.sourceBrowser)  /* HTTP request for source browser */
                {
                    httpChannel.loadFlags = Components.interfaces.nsICachingChannel.LOAD_ONLY_FROM_CACHE;
                    httpChannel.cancel(Components.results.NS_ERROR_FAILURE);
                }
            }
        }
    },

    /********************************************************************/

    /* Load and Unload functions */

    onLoad: function()
    {
        var printEditTB,navToolbox,printPreviewTB;
        
        /* Determine Firefox version */
        
        printEdit.ffVersion = "3.6";
        if (printEdit.versionComparator.compare(printEdit.appInfo.version,"4.0b1") >= 0) printEdit.ffVersion = "4.0";
        
        document.getElementById("main-window").setAttribute("printedit-ffversion",printEdit.ffVersion);
        
        /* Relocate print edit toolbar */
        
        printEditTB = document.getElementById("printedit-toolbar");
        
        navToolbox = getNavToolbox();
        navToolbox.parentNode.insertBefore(printEditTB.cloneNode(true),navToolbox);
        
        printEditTB.parentNode.removeChild(printEditTB);
        
        /* Redefine PrintUtils.enterPrintPreview */
        
        PrintUtils.old_enterPrintPreview = PrintUtils.enterPrintPreview;
        
        PrintUtils.enterPrintPreview = function new_enterPrintPreview()
        {
            /* Determine source and preview browsers */
            
            if (printEdit.ffVersion == "3.6")
            {
                printEdit.previewBrowser = gBrowser.mCurrentBrowser;
                printEdit.sourceBrowser = gBrowser.mCurrentBrowser;
            }
            else if (printEdit.ffVersion == "4.0")
            {
                printEdit.previewBrowser = PrintPreviewListener.getPrintPreviewBrowser();
                printEdit.sourceBrowser = PrintPreviewListener.getSourceBrowser();
            }
            
            /* Display using screen stylesheets for print device */
            
            if (printEdit.prefs.getIntPref("display-mediaprint") == 2) printEdit.displayCSSMediaPrint(2);
            
            /* Call original enterPrintPreview */
            
            PrintUtils.old_enterPrintPreview();
            
            if (document.getElementById("print-preview-toolbar") == null) return;  /* enter print preview failed */
            
            /* Enter print edit mode if button menu or context menu command */
            
            if (printEdit.menuPrintEdit)
            {
                printEdit.menuPrintEdit = false;
                
                printEdit.enterPrintEdit();
            }
            
            /* Enable debug */
            
            if (document.getElementById("printedit-debug-info") != null)
                document.getElementById("status-bar").setAttribute("hidden","false");
        }
        
        /* Redefine PrintUtils.exitPrintPreview */
        
        PrintUtils.old_exitPrintPreview = PrintUtils.exitPrintPreview;
        
        PrintUtils.exitPrintPreview = function new_exitPrintPreview()
        {
            /* If print edit mode has been entered */
            
            if (printEdit.curMode > 0)
            {
                /* Exit print edit mode */
                
                if (printEdit.curMode == 2) printEdit.exitPrintEdit();
                
                /* Undo all commands */
                
                while (printEdit.cmdNumber >= 0)
                {
                    printEdit.deselectElements();
                    printEdit.cmdNumber--;
                }
                
                printEdit.cmdNumber = -1;
                
                printEdit.undoCommands(printEdit.sourceBrowser.contentDocument.body);
                
                printEdit.cmdNumber = 0;
                
                /* Enable quick find toolbar */
                
                gFindBar.style.removeProperty("display");
                
                /* Allow source page loads and refreshes */
                
                printEdit.httpObserver.unregister();
                
                /* Set initial print preview mode */
                
                printEdit.curMode = 0;
                
                /* Call original exitPrintPreview */
                
                PrintUtils.old_exitPrintPreview();
                
                /* Reload source page */
                
                printEdit.sourceBrowser.reload();
            }
            else
            {
                /* Call original exitPrintPreview */
                
                PrintUtils.old_exitPrintPreview();
            }
        }
        
        /* Redefine PrintUtils.onKeyPressPP */
        
        PrintUtils.old_onKeyPressPP = PrintUtils.onKeyPressPP;
        
        PrintUtils.onKeyPressPP = function new_onKeyPressPP(event)
        {
            if (event.keyCode == event.DOM_VK_F1 || event.keyCode == event.DOM_VK_F5 || event.keyCode == event.DOM_VK_F11 ||
                event.keyCode == event.DOM_VK_HOME || event.keyCode == event.DOM_VK_LEFT || event.keyCode == event.DOM_VK_RIGHT ||
                event.keyCode == event.DOM_VK_BACK_SPACE)
            {
                /* Catch F1, F5, F11, Alt+Home, Alt+LeftArrow, Alt+RightArrow, Backspace, Shift+Backspace */
                
                event.preventDefault();
                event.stopPropagation();
            }
            else if (document.getElementById("printedit-options-panel").state == "open" &&
                     (event.keyCode == event.DOM_VK_ESCAPE || event.keyCode == event.DOM_VK_ENTER || event.keyCode == event.DOM_VK_RETURN))
            {
                if (event.keyCode == event.DOM_VK_ESCAPE) printEdit.onOptionsClose();
                else printEdit.onOptionsOkay();
                
                event.preventDefault();
                event.stopPropagation();
            }
            else PrintUtils.old_onKeyPressPP(event);
        }
    },
    
    onUnload: function()
    {
    },

    /********************************************************************/

    /* Enter print edit mode */

    enterPrintEdit: function()
    {
        var webBrowserPrint,printPreviewTB,printEditTB,selectIndex;
        
        /* First entry into print edit mode */
        
        if (printEdit.curMode == 0)
        {
            /* Display using print stylesheets for screen device */
            
            if (printEdit.prefs.getIntPref("display-mediaprint") == 0) printEdit.displayCSSMediaPrint(0);
            
            /* Initialise for print edit mode */
            
            printEdit.cmdNumber = 0;
            printEdit.selectCount[0] = 0;
            printEdit.selectElement[0] = new Array();
            
            /* Prevent source page loads and refreshes */
            
            printEdit.httpObserver.register();
            
            /* Disable active elements in source page after http disabled */
            
            printEdit.disableActiveElements(printEdit.sourceBrowser.contentDocument.body);
            
            /* Disable quick find toolbar */
            
            if (printEdit.ffVersion == "4.0" && !gFindBarInitialized)
            {
                gFindBar.open();
                gFindBar.close();
            }
            
            gFindBar.style.setProperty("display","none","");
        }
        
        /* Replicate code from exitPrintPreview() in PrintUtils.js */
        
        if (printEdit.ffVersion == "3.6")
        {
            /* Exit print preview mode */
            
            webBrowserPrint = PrintUtils.getWebBrowserPrint();
            webBrowserPrint.exitPrintPreview();
            
            /* Remove print preview toolbar */
            
            printPreviewTB = document.getElementById("print-preview-toolbar");
            getNavToolbox().parentNode.removeChild(printPreviewTB);
            
            /* Set focus on content */
            
            window.content.focus();
        }
        else if (printEdit.ffVersion == "4.0")
        {        
            /* Enter print preview mode */
            
            webBrowserPrint = PrintUtils.getPrintPreview();
            webBrowserPrint.exitPrintPreview();
            
            /* Remove print preview toolbar */
            
            printPreviewTB = document.getElementById("print-preview-toolbar");
            PrintUtils._callback.getNavToolbox().parentNode.removeChild(printPreviewTB);
            
            /* Select original source tab */
            
            gBrowser.selectedTab = PrintPreviewListener._tabBeforePrintPreview;
            
            /* Set focus on content */
            
            window.content.focus();
        }
        
        /* Create print edit toolbar */
        
        printEditTB = document.getElementById("printedit-toolbar");
        printEditTB.setAttribute("hidden","false");
        
        /* Disable buttons */
        
        printEdit.disableEditButtons();
        if (printEdit.cmdNumber == 0) printEdit.disableUndoButtons();
        
        /* Default highlight element */
        
        printEdit.highlightElement = printEdit.sourceBrowser.contentDocument.body;
        
        /* Create highlight box */
        
        printEdit.highlightBox = printEdit.createBox("highlight");
        
        /* Create and show select boxes */
        
        for (selectIndex = 0; selectIndex < printEdit.selectCount[printEdit.cmdNumber]; selectIndex++)
        {
            printEdit.selectBox[selectIndex] = printEdit.createBox("select");
            printEdit.showBox(printEdit.selectBox[selectIndex],printEdit.selectElement[printEdit.cmdNumber][selectIndex]);
        }
        
        if (printEdit.selectCount[printEdit.cmdNumber] > 0) printEdit.enableEditButtons();
        
        /* Add mouse listeners */
        
        printEdit.sourceBrowser.contentDocument.body.addEventListener("mouseover",printEdit.onMouseOverBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.addEventListener("mouseout",printEdit.onMouseOutBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.addEventListener("mousedown",printEdit.onMouseDownBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.addEventListener("mouseup",printEdit.onMouseUpBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.addEventListener("click",printEdit.onClickBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.addEventListener("dblclick",printEdit.onDblClickBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.addEventListener("dragstart",printEdit.onDragStartBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.addEventListener("contextmenu",printEdit.onContextMenuBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.addEventListener("DOMMouseScroll",printEdit.onMouseScrollBrowser,true);
        
        /* Add window resize listener */
        
        window.addEventListener("resize",printEdit.onResizeWindow,false);
        
        /* Add options panel popup listeners */
        
        document.getElementById("printedit-options-panel").addEventListener("popupshowing",printEdit.onOptionsShowing,false);
        
        /* Set print edit mode */
        
        printEdit.curMode = 2;
    },

    /********************************************************************/

    /* Exit print edit mode - return to print preview mode */

    exitPrintEdit: function()
    {
        var printEditTB,webBrowserPrint,printSettings,XUL_NS,printPreviewTB,navToolbox,browser;
        
        /* Replicate code from enterPrintPreview() in PrintUtils.js */
        
        if (printEdit.ffVersion == "3.6")
        {
            /* Enter print preview mode */
            
            printSettings   = PrintUtils.getPrintSettings();
            webBrowserPrint = PrintUtils.getWebBrowserPrint();
            try
            {
                webBrowserPrint.printPreview(printSettings,null,PrintUtils._webProgressPP.value);
            }
            catch (e)
            {
                return;
            }
            
            /* Create print preview toolbar */
            
            XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
            
            printPreviewTB = document.createElementNS(XUL_NS,"toolbar");
            printPreviewTB.setAttribute("printpreview", true);
            printPreviewTB.setAttribute("id","print-preview-toolbar");
            printPreviewTB.className = "toolbar-primary";
            
            navToolbox = getNavToolbox();
            navToolbox.parentNode.insertBefore(printPreviewTB,navToolbox);
            
            /* Set focus on content */
            
            window.content.focus();
        }
        else if (printEdit.ffVersion == "4.0")
        {        
            /* Enter print preview mode */
            
            printSettings   = PrintUtils.getPrintSettings();
            webBrowserPrint = PrintUtils.getPrintPreview();
            try
            {
                webBrowserPrint.printPreview(printSettings,PrintUtils._sourceBrowser.contentWindow,PrintUtils._webProgressPP.value);
            }
            catch (e)
            {
                return;
            }
            
            /* Create print preview toolbar */
            
            XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
            
            printPreviewTB = document.createElementNS(XUL_NS,"toolbar");
            printPreviewTB.setAttribute("printpreview", true);
            printPreviewTB.setAttribute("id","print-preview-toolbar");
            printPreviewTB.className = "toolbar-primary";
            
            navToolbox = PrintUtils._callback.getNavToolbox();
            navToolbox.parentNode.insertBefore(printPreviewTB,navToolbox);
            
            /* Select print preview tab */
            
            gBrowser.selectedTab = PrintPreviewListener._printPreviewTab;
            
            /* Set focus on content */
            
            browser = PrintUtils._callback.getPrintPreviewBrowser();
            browser.collapsed = false;
            browser.contentWindow.focus();
        }
        
        /* Workaround - print preview is initially displayed 1152 pixels wide */
        
        printEdit.previewBrowser.style.setProperty("width","0px","");
        printEdit.previewBrowser.style.setProperty("height","0px","");
        window.setTimeout(printEdit.exitPrintEditFix,100);
        
        /* Remove options panel popup listener */
        
        document.getElementById("printedit-options-panel").removeEventListener("popupshowing",printEdit.onOptionsShowing,false);
        
        /* Remove window resize listener */
        
        window.removeEventListener("resize",printEdit.onResizeWindow,false);
        
        /* Remove mouse listeners */
        
        printEdit.sourceBrowser.contentDocument.body.removeEventListener("mouseover",printEdit.onMouseOverBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.removeEventListener("mouseout",printEdit.onMouseOutBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.removeEventListener("mousedown",printEdit.onMouseDownBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.removeEventListener("mouseup",printEdit.onMouseUpBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.removeEventListener("click",printEdit.onClickBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.removeEventListener("dblclick",printEdit.onDblClickBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.removeEventListener("dragstart",printEdit.onDragStartBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.removeEventListener("contextmenu",printEdit.onContextMenuBrowser,true);
        printEdit.sourceBrowser.contentDocument.body.removeEventListener("DOMMouseScroll",printEdit.onMouseScrollBrowser,true);
        
        /* Destroy select boxes */
        
        printEdit.destroyBoxes();
        
        /* Destroy highlight box */
        
        printEdit.destroyBox(printEdit.highlightBox);
        printEdit.highlightBox = null;
        
        /* Remove print edit toolbar */
        
        printEditTB = document.getElementById("printedit-toolbar");
        printEditTB.setAttribute("hidden","true");
        
        /* Set print preview mode */
        
        printEdit.curMode = 1;
    },
    
    exitPrintEditFix: function()
    {
        printEdit.previewBrowser.style.removeProperty("width");
        printEdit.previewBrowser.style.removeProperty("height");
    },

    /********************************************************************/

    /* Create and display boxes functions*/

    createBox: function(type)
    {
        var i,box,zindex;
        
        if (type == "highlight")
        {
            zindex = "9999";
        }
        else
        {
            zindex = "9998";
        }
        
        box = new printEdit.Box;
        
        box.container =  printEdit.sourceBrowser.contentDocument.createElement("div");
        box.container.setAttribute("class","printedit-" + type + "box");
        box.container.style.setProperty("position","absolute","");
        box.container.style.setProperty("width","0px","");
        box.container.style.setProperty("height","0px","");
        box.container.style.setProperty("left","0px","");
        box.container.style.setProperty("top","0px","");
        printEdit.sourceBrowser.contentDocument.body.appendChild(box.container);
        
        for (i = 0; i <= 3; i++)
        {
            box.sides[i] = printEdit.sourceBrowser.contentDocument.createElement("div");
            box.sides[i].style.setProperty("position","absolute","");
            box.sides[i].style.setProperty("width","0px","");
            box.sides[i].style.setProperty("height","0px","");
            box.sides[i].style.setProperty("left","0px","");
            box.sides[i].style.setProperty("top","0px","");
            box.sides[i].style.setProperty("z-index",zindex,"");
            box.sides[i].style.setProperty("display","none","");
            box.container.appendChild(box.sides[i]);
        }
        
        return box;
    },

    destroyBox: function(box)
    {
        var i;
        
        for (i = 0; i <= 3; i++) box.container.removeChild(box.sides[i]);
        
        box.sides.length = 0;
        
        printEdit.sourceBrowser.contentDocument.body.removeChild(box.container);
        
        delete box;
    },

    showBox: function(box,element)
    {
        var i,color,width,style;
        
        if (box.container.getAttribute("class") == "printedit-highlightbox")
        {
            color = printEdit.prefs.getCharPref("highlight-color");
            width = printEdit.prefs.getIntPref("highlight-width");
            style = "solid";
        }
        else
        {
            color = printEdit.prefs.getCharPref("select-color");
            width = printEdit.prefs.getIntPref("select-width");
            style = "dashed";
        }
        
        for (i = 0; i <= 3; i++) box.sides[i].style.removeProperty("display");
        
        box.sides[0].style.setProperty("height",element.offsetHeight + "px","");
        box.sides[0].style.setProperty("left",printEdit.calculateX(element) + "px","");
        box.sides[0].style.setProperty("top",printEdit.calculateY(element) + "px","");
        box.sides[0].style.setProperty("border-left",width + "px " + style + " " + color,"");
        
        box.sides[1].style.setProperty("height",element.offsetHeight + "px","");
        box.sides[1].style.setProperty("left",(printEdit.calculateX(element)+element.offsetWidth-width) + "px","");
        box.sides[1].style.setProperty("top",printEdit.calculateY(element) + "px","");
        box.sides[1].style.setProperty("border-right",width + "px " + style + " " + color,"");
       
        box.sides[2].style.setProperty("width",element.offsetWidth + "px","");
        box.sides[2].style.setProperty("left",printEdit.calculateX(element) + "px","");
        box.sides[2].style.setProperty("top",printEdit.calculateY(element) + "px","");
        box.sides[2].style.setProperty("border-top",width + "px " + style + " " + color,"");
        
        box.sides[3].style.setProperty("width",element.offsetWidth + "px","");
        box.sides[3].style.setProperty("left",printEdit.calculateX(element) + "px","");
        box.sides[3].style.setProperty("top",(printEdit.calculateY(element)+element.offsetHeight-width) + "px","");
        box.sides[3].style.setProperty("border-bottom",width + "px " + style + " " + color,"");
    },

    hideBox: function(box)
    {
        var i;
        
        for (i = 0; i <= 3; i++) box.sides[i].style.setProperty("display","none","");
    },

    calculateX: function(element)
    {
        var x;

        x = 0;
        while (element.offsetParent)
        {
            x += element.offsetLeft;
            element = element.offsetParent;
        }
   
        return x;
    },

    calculateY: function(element)
    {
        var y;
        
        y = 0;
        while (element.offsetParent)
        {
            y += element.offsetTop;
            element = element.offsetParent;
        }
          
        return y;
    },

    /********************************************************************/

    /* Select management functions */

    destroyBoxes: function()
    {
        var selectIndex;
        
        for (selectIndex = 0; selectIndex < printEdit.selectCount[printEdit.cmdNumber]; selectIndex++)
            printEdit.destroyBox(printEdit.selectBox[selectIndex]);
            
        printEdit.selectBox.length = 0;
    },

    deselectElements: function()
    {
        var selectIndex;
        
        printEdit.selectCount[printEdit.cmdNumber] = 0;
        printEdit.selectElement[printEdit.cmdNumber].length = 0;
    },

    nextCommand: function()
    {
        printEdit.cmdNumber++;
        printEdit.selectCount[printEdit.cmdNumber] = 0;
        printEdit.selectElement[printEdit.cmdNumber] = new Array();
    },

    /********************************************************************/

    /* Button enable/disable functions */

    enableEditButtons: function()
    {
        var button;
    
        button = document.getElementById("printedit-deselect");
        button.setAttribute("disabled","false");
        button = document.getElementById("printedit-format");
        button.setAttribute("disabled","false");
        button = document.getElementById("printedit-hide");
        button.setAttribute("disabled","false");
        button = document.getElementById("printedit-delete");
        button.setAttribute("disabled","false");
        button = document.getElementById("printedit-hideexcept");
        button.setAttribute("disabled","false");
        button = document.getElementById("printedit-deleteexcept");
        button.setAttribute("disabled","false");
    },

    disableEditButtons: function()
    {
        var button;
    
        button = document.getElementById("printedit-deselect");
        button.setAttribute("disabled","true");
        button = document.getElementById("printedit-format");
        button.setAttribute("disabled","true");
        button = document.getElementById("printedit-hide");
        button.setAttribute("disabled","true");
        button = document.getElementById("printedit-delete");
        button.setAttribute("disabled","true");
        button = document.getElementById("printedit-hideexcept");
        button.setAttribute("disabled","true");
        button = document.getElementById("printedit-deleteexcept");
        button.setAttribute("disabled","true");
    },

    enableUndoButtons: function()
    {
        var button;
    
        button = document.getElementById("printedit-undo");
        button.setAttribute("disabled","false");
        button = document.getElementById("printedit-undoall");
        button.setAttribute("disabled","false");
    },

    disableUndoButtons: function()
    {
        var button;
    
        button = document.getElementById("printedit-undo");
        button.setAttribute("disabled","true");
        button = document.getElementById("printedit-undoall");
        button.setAttribute("disabled","true");
    },

    /********************************************************************/

    /* Recursive format/hide/delete/undo functions */

    formatElement: function(element)
    {
        var i,formatString;
        
        if (element.getAttribute("class") == "printedit-highlightbox") return;
        if (element.getAttribute("class") == "printedit-selectbox") return;
        
        for (i = 0; i < element.children.length; i++)
            printEdit.formatElement(element.children[i]);
        
        if (!element.hasAttribute("printedit-format"))
        {
            /* Based on Eric Meyer's CSS Reset */
            
            formatString = printEdit.cmdNumber + ";";
            if (printEdit.prefs.getBoolPref("format-margin"))
            {
                formatString += "margin:" + element.style.margin + ";";
                element.style.setProperty("margin","0","");
            }
            if (printEdit.prefs.getBoolPref("format-padding"))
            {
                formatString += "padding:" + element.style.padding + ";";
                element.style.setProperty("padding","0","");
            }
            if (printEdit.prefs.getBoolPref("format-border"))
            {
                formatString += "border:" + element.style.border + ";";
                element.style.setProperty("border","0","");
            }
            if (printEdit.prefs.getBoolPref("format-outline"))
            {
                formatString += "outline:" + element.style.outline + ";";
                element.style.setProperty("outline","0","");
            }
            if (printEdit.prefs.getBoolPref("format-verticalalign"))
            {
                formatString += "vertical-align:" + element.style.verticalAlign + ";";
                element.style.setProperty("vertical-align","baseline","");
            }
            if (printEdit.prefs.getBoolPref("format-textalign"))
            {
                formatString += "text-align:" + element.style.textAlign + ";";
                element.style.setProperty("text-align","left","");
            }
            if (printEdit.prefs.getBoolPref("format-lineheight"))
            {
                formatString += "line-height:" + element.style.lineHeight + ";";
                element.style.setProperty("line-height","1","");
            }
            if (printEdit.prefs.getBoolPref("format-liststyle"))
            {
                formatString += "list-style:" + element.style.listStyle + ";";
                element.style.setProperty("list-style","none","");
            }
            element.setAttribute("printedit-format",formatString);
            
            /*        
            element.style.setProperty("font-weight","inherit","");
            element.style.setProperty("font-family","inherit","");
            element.style.setProperty("font-style","inherit","");
            element.style.setProperty("font-size","100%","");
            
            element.style.setProperty("background","white","");
            element.style.setProperty("color","black","");
            
            element.style.setProperty("border-collapse","separate","");
            element.style.setProperty("border-spacing","0","");
            */
        }
    },

    hideExcept: function(element)
    {
        var i,selectIndex,keepNode;
        
        keepNode = false;
        
        if (element.getAttribute("class") == "printedit-highlightbox") return true;
        if (element.getAttribute("class") == "printedit-selectbox") return true;
        
        for (selectIndex = 0; selectIndex < printEdit.selectCount[printEdit.cmdNumber]; selectIndex++)
            if (printEdit.selectElement[printEdit.cmdNumber][selectIndex] == element) return true;
        
        for (i = 0; i < element.children.length; i++)
            if (printEdit.hideExcept(element.children[i])) keepNode = true;
        
        if (!keepNode)
        {
            if (!element.hasAttribute("printedit-hide"))
            {
                if (printEdit.prefs.getBoolPref("hide-select"))
                {
                    element.setAttribute("printedit-hide",printEdit.cmdNumber + ";opacity:" + element.style.opacity + ";");
                    element.style.setProperty("opacity","0","");
                }
                else
                {
                    element.setAttribute("printedit-hide",printEdit.cmdNumber + ";visibility:" + element.style.visibility + ";");
                    element.style.setProperty("visibility","hidden","");
                }
            }
        }
        
        return keepNode;
    },

    deleteExcept: function(element)
    {
        var i,selectIndex,keepNode;
        
        keepNode = false;
        
        if (element.getAttribute("class") == "printedit-highlightbox") return true;
        if (element.getAttribute("class") == "printedit-selectbox") return true;
        
        for (selectIndex = 0; selectIndex < printEdit.selectCount[printEdit.cmdNumber]; selectIndex++)
            if (printEdit.selectElement[printEdit.cmdNumber][selectIndex] == element) return true;
        
        for (i = 0; i < element.children.length; i++)
            if (printEdit.deleteExcept(element.children[i])) keepNode = true;
        
        if (!keepNode)
        {
            if (!element.hasAttribute("printedit-delete"))
            {
                element.setAttribute("printedit-delete",printEdit.cmdNumber + ";display:" + element.style.display + ";");
                element.style.setProperty("display","none","");
            }
        }
        
        return keepNode;
    },

    undoCommands: function(element)
    {
        var i;
        var undoInfo = new Array();
        var propertyField = new Array();
        
        if (element.getAttribute("class") == "printedit-highlightbox") return;
        if (element.getAttribute("class") == "printedit-selectbox") return;
        
        for (i = 0; i < element.children.length; i++)
            printEdit.undoCommands(element.children[i]);
        
        if (element.hasAttribute("printedit-format"))
        {
            undoInfo = element.getAttribute("printedit-format").split(";");
            
            if (printEdit.cmdNumber == -1 || printEdit.cmdNumber == +undoInfo[0])
            {
                for (i = 1; i < undoInfo.length; i++)
                {
                    propertyField = undoInfo[i].split(":");
                    if (propertyField[1] == "") element.style.removeProperty(propertyField[0]);
                    else element.style.setProperty(propertyField[0],propertyField[1],"");
                }
                
                element.removeAttribute("printedit-format");
            }
        }
        
        if (element.hasAttribute("printedit-hide"))
        {
            undoInfo = element.getAttribute("printedit-hide").split(";");
            
            if (printEdit.cmdNumber == -1 || printEdit.cmdNumber == +undoInfo[0])
            {
                propertyField = undoInfo[1].split(":");
                if (propertyField[1] == "") element.style.removeProperty(propertyField[0]);
                else element.style.setProperty(propertyField[0],propertyField[1],"");
                
                element.removeAttribute("printedit-hide");
            }
        }
        
        if (element.hasAttribute("printedit-delete"))
        {
            undoInfo = element.getAttribute("printedit-delete").split(";");
            
            if (printEdit.cmdNumber == -1 || printEdit.cmdNumber == +undoInfo[0])
            { 
                propertyField = undoInfo[1].split(":");
                if (propertyField[1] == "") element.style.removeProperty(propertyField[0]);
                else element.style.setProperty(propertyField[0],propertyField[1],"");
                
                element.removeAttribute("printedit-delete");
            }
        }
    },

    /********************************************************************/

    /* Active elements disable function */

    disableActiveElements: function(element)
    {
        var i,iframe,object,embed,parent;
        
        if (element.getAttribute("class") == "printedit-highlightbox") return;
        if (element.getAttribute("class") == "printedit-selectbox") return;
        
        if (element.localName == "input" || element.localName == "button" || element.localName == "textarea" ||
            element.localName == "select" || element.localName == "optgroup" || element.localName == "option")
        {
            /* Disable <input>/<button>/<textarea>/<select>/<optgroup>/<option> elements */
            
            if (!element.hasAttribute("disabled"))
            {
                element.setAttribute("printedit-disabled","");
                element.setAttribute("disabled","disabled");
            }
        }
        else if (element.localName == "iframe")
        {
            /* Disable <iframe> element by cloning after http disabled so element cannot reload */
            
            iframe = element.cloneNode(true);
            
            parent = element.parentNode;
            parent.replaceChild(iframe,element);
            
            return;
        }
        else if (element.localName == "object")
        {
            /* Disable <object> element by cloning after http disabled so element cannot reload */
            /* This also suppresses hand cursor */
            
            object = element.cloneNode(true);
            
            parent = element.parentNode;
            parent.replaceChild(object,element);
            
            return;
        }
        else if (element.localName == "embed")
        {
            /* Disable <embed> element by cloning after http disabled so element cannot reload */
            /* This also suppresses hand cursor */
            
            embed = element.cloneNode(true);
            
            parent = element.parentNode;
            parent.replaceChild(embed,element);
            
            return;
        }
       
        for (i = 0; i < element.children.length; i++)
            printEdit.disableActiveElements(element.children[i]);
    },

    /********************************************************************/

    /* CSS Print Stylesheets display function */

    displayCSSMediaPrint: function(mediaPrint)
    {
        var i,j,documentList,pageDocument,styleSheetList,styleSheet,media;
        
        documentList = printEdit.getDocuments(printEdit.sourceBrowser.contentWindow);
        
        for (i = 0; i < documentList.length; i++)
        {
            pageDocument = documentList[i];
            styleSheetList = pageDocument.styleSheets;
            
            for (j = 0; j < styleSheetList.length; j++)
            {
                styleSheet = styleSheetList[j];
                
                if (printEdit.isValidStyleSheet(styleSheet) && !printEdit.isAlternateStyleSheet(styleSheet))
                {
                    media = styleSheet.media;
                    
                    /* Reset stylesheet media types */
                    
                    if (printEdit.isMediaStyleSheet(styleSheet,"printedit-append-screen"))
                    {
                        media.deleteMedium("screen");
                        media.deleteMedium("printedit-append-screen");
                    }
                    
                    if (printEdit.isMediaStyleSheet(styleSheet,"printedit-delete-screen"))
                    {
                        media.appendMedium("screen");
                        media.deleteMedium("printedit-delete-screen");
                    }
                    
                    if (printEdit.isMediaStyleSheet(styleSheet,"printedit-append-print"))
                    {
                        media.deleteMedium("print");
                        media.deleteMedium("printedit-append-print");
                    }
                    
                    if (printEdit.isMediaStyleSheet(styleSheet,"printedit-delete-print"))
                    {
                        media.appendMedium("print");
                        media.deleteMedium("printedit-delete-print");
                    }
                    
                    /* Use print stylesheet for screen device */
                    
                    if (mediaPrint == 0)
                    {
                        if (printEdit.isMediaStyleSheet(styleSheet,"print"))
                        {
                            if (!printEdit.isMediaStyleSheet(styleSheet,"screen"))
                            {
                                media.appendMedium("screen");
                                media.appendMedium("printedit-append-screen");
                            }
                        }
                        else
                        {
                            if (printEdit.isMediaStyleSheet(styleSheet,"screen"))
                            {
                                media.deleteMedium("screen");
                                media.appendMedium("printedit-delete-screen");
                            }
                        }
                    }
                    
                    /* Use screen stylesheet for print device */
                    
                    if (mediaPrint == 2)
                    {
                        if (printEdit.isMediaStyleSheet(styleSheet,"screen"))
                        {
                            if (!printEdit.isMediaStyleSheet(styleSheet,"print"))
                            {
                                media.appendMedium("print");
                                media.appendMedium("printedit-append-print");
                            }
                        }
                        else
                        {
                            if (printEdit.isMediaStyleSheet(styleSheet,"print"))
                            {
                                media.deleteMedium("print");
                                media.appendMedium("printedit-delete-print");
                            }
                        }
                    }
                    
                    /* Force stylesheet to be re-applied */
                    
                    styleSheet.disabled = true;
                    styleSheet.disabled = false;
                }
            }
        }
    },

    getDocuments: function(frame)
    {
        var i,frames;
        var documents = new Array();
        
        frames = frame.frames;
        
        if (frame.document) documents.push(frame.document);
        
        for (var i = 0; i < frames.length; i++)
        {
            documents = documents.concat(printEdit.getDocuments(frames[i]));
        }
        
        return documents;
    },

    isValidStyleSheet: function(styleSheet)
    {
        var styleSheetHref;
        
        styleSheetHref = styleSheet.href;
        
        /* If the style sheet href is set and this is not an internal style sheet */
        
        if (styleSheetHref && styleSheetHref.indexOf("about:PreferenceStyleSheet") != 0 && 
            styleSheetHref.indexOf("chrome://") != 0 && styleSheetHref.indexOf("data:text/css") != 0 &&
            styleSheetHref.indexOf("jar:file://") != 0 && styleSheetHref.indexOf("resource://") != 0) return true;
        
        /* If the style sheet href is not set */
        
        if (!styleSheetHref) return true;
        
        return false;
    },

    isAlternateStyleSheet: function(styleSheet)
    {
        var ownerNode;
        
        ownerNode = styleSheet.ownerNode;
        
        if (ownerNode)
        {
            if (ownerNode.nodeType == Components.interfaces.nsIDOMNode.PROCESSING_INSTRUCTION_NODE)
            {
                if (ownerNode.data.indexOf('alternate="yes"') != -1) return true;
            }
            else
            {
                if (ownerNode.hasAttribute("rel") && ownerNode.getAttribute("rel").toLowerCase() == "alternate stylesheet") return true;
            }
        }
        
        return false;
    },

    isMediaStyleSheet: function(styleSheet,matchMediaType)
    {
        var i,media,mediaType;
        
        media = styleSheet.media;
        
        for (i = 0; i < media.length; i++)
        {
            mediaType = media.item(i).toLowerCase();
            if (mediaType == matchMediaType) return true;
        }
        
        return false;
    },

    /********************************************************************/

    /* Event handlers */

    onMouseOverBrowser: function(event)
    {
        var i,element,selectIndex;
        
        element = event.originalTarget;
        
        element.style.setProperty("cursor","default","");
        
        if (element.localName == "html") element = printEdit.sourceBrowser.contentDocument.body;
        
        for (i = 0; i <= 3; i++)
            if (event.originalTarget == printEdit.highlightBox.sides[i]) element = printEdit.highlightElement;
        
        for (selectIndex = 0; selectIndex < printEdit.selectCount[printEdit.cmdNumber]; selectIndex++)
            for (i = 0; i <= 3; i++)
                if (event.originalTarget == printEdit.selectBox[selectIndex].sides[i]) element = printEdit.selectElement[printEdit.cmdNumber][selectIndex];
        
        printEdit.highlightElement = element;
        
        printEdit.showBox(printEdit.highlightBox,element);
        printEdit.showBox(printEdit.highlightBox,element);  /* occasionally the box sides are in the wrong position after the first call !!! */

        event.preventDefault();
        event.stopPropagation();
    },    

    onMouseOutBrowser: function(event)
    {
        /* necessary to handle cursor moving off content area */
        
        element = event.originalTarget;

        element.style.removeProperty("cursor");
        
        printEdit.hideBox(printEdit.highlightBox);

        event.preventDefault();
        event.stopPropagation();
    },

    onMouseDownBrowser: function(event)
    {
        /* prevents mousedown actions (if there are any) */
        
        event.preventDefault();
        event.stopPropagation();
    },

    onMouseUpBrowser: function(event)
    {
        /* prevents mouseup activation of iframe, object and embed links */
        
        event.preventDefault();
        event.stopPropagation();
    },

    onClickBrowser: function(event)
    {
        /* handles selection and de-selection of elements */
        /* prevents click activation of links */
        
        var selectIndex;
        
        if (event.button == 0)
        {
            for (selectIndex = 0; selectIndex < printEdit.selectCount[printEdit.cmdNumber]; selectIndex++)
                if (printEdit.selectElement[printEdit.cmdNumber][selectIndex] == printEdit.highlightElement) break;
                
            if (selectIndex < printEdit.selectCount[printEdit.cmdNumber])  /* remove selection box */
            {
                printEdit.destroyBox(printEdit.selectBox[selectIndex]);
                printEdit.hideBox(printEdit.highlightBox);
                
                for (i = selectIndex; i < printEdit.selectCount[printEdit.cmdNumber]-1; i++)
                {
                    printEdit.selectElement[printEdit.cmdNumber][i] = printEdit.selectElement[printEdit.cmdNumber][i+1];
                    printEdit.selectBox[i] = printEdit.selectBox[i+1];
                }

                printEdit.selectCount[printEdit.cmdNumber]--;
                printEdit.selectElement[printEdit.cmdNumber].length--;
                printEdit.selectBox.length--;
                
                if (printEdit.selectCount[printEdit.cmdNumber] == 0) printEdit.disableEditButtons();
            }
            else  /* add selection box */
            {
                printEdit.selectElement[printEdit.cmdNumber][printEdit.selectCount[printEdit.cmdNumber]] = printEdit.highlightElement;
                printEdit.selectBox[printEdit.selectCount[printEdit.cmdNumber]] = printEdit.createBox("select");
                
                printEdit.showBox(printEdit.selectBox[printEdit.selectCount[printEdit.cmdNumber]],printEdit.highlightElement);
                printEdit.hideBox(printEdit.highlightBox);
                
                printEdit.selectCount[printEdit.cmdNumber]++;

                printEdit.enableEditButtons();
            }
        }
        
        event.preventDefault();
        event.stopPropagation();
    },

    onDblClickBrowser: function(event)
    {
        /* prevents double click activation of links */
        
        event.preventDefault();
        event.stopPropagation();
    },

    onDragStartBrowser: function(event)
    {
        /* prevents drag-and-drop activation of links */
        
        event.preventDefault();
        event.stopPropagation();
    },

    onContextMenuBrowser: function(event)
    {
        /* prevents context menu popup */
        
        event.preventDefault();
        event.stopPropagation();
    },

    onMouseScrollBrowser: function(event)
    {
        /* prevents scroll forward and backward navigation */
        
        if (event.shiftKey)
        {   
            event.preventDefault();
            event.stopPropagation();
        }
    },

    onResizeWindow: function(event)
    {
        /* handles window re-size and re-positions select boxes */
        
        var selectIndex;
        
        for (selectIndex = 0; selectIndex < printEdit.selectCount[printEdit.cmdNumber]; selectIndex++)
            printEdit.showBox(printEdit.selectBox[selectIndex],printEdit.selectElement[printEdit.cmdNumber][selectIndex]);
    },

    onOptionsShowing: function(event)
    {
        var button,panel;
        
        if (event.originalTarget.id != "printedit-options-panel") return;
        
        /* Position panel */
        
        button = document.getElementById("printedit-options");
        panel = document.getElementById("printedit-options-panel");
        if (printEdit.ffVersion == "3.6") panel.moveTo(button.boxObject.screenX-254,button.boxObject.screenY+28);
        else if (printEdit.ffVersion == "4.0") panel.moveTo(button.boxObject.screenX-254,button.boxObject.screenY+30);
        
        /* Display Appearance Options */
        
        document.getElementById("printedit-display-mediaprint").selectedIndex = printEdit.prefs.getIntPref("display-mediaprint");
        
        /* Highlight Box Options */
        
        document.getElementById("printedit-highlight-color").color = printEdit.prefs.getCharPref("highlight-color");
        document.getElementById("printedit-highlight-width").selectedIndex = printEdit.prefs.getIntPref("highlight-width")-1;
        
        /* Select Box Options */
        
        document.getElementById("printedit-select-color").color = printEdit.prefs.getCharPref("select-color");
        document.getElementById("printedit-select-width").selectedIndex = printEdit.prefs.getIntPref("select-width")-1;
        
        /* Format Properties Options */
        
        document.getElementById("printedit-format-margin").checked = printEdit.prefs.getBoolPref("format-margin");
        document.getElementById("printedit-format-padding").checked = printEdit.prefs.getBoolPref("format-padding");
        document.getElementById("printedit-format-border").checked = printEdit.prefs.getBoolPref("format-border");
        document.getElementById("printedit-format-outline").checked = printEdit.prefs.getBoolPref("format-outline");
        document.getElementById("printedit-format-verticalalign").checked = printEdit.prefs.getBoolPref("format-verticalalign");
        document.getElementById("printedit-format-textalign").checked = printEdit.prefs.getBoolPref("format-textalign");
        document.getElementById("printedit-format-lineheight").checked = printEdit.prefs.getBoolPref("format-lineheight");
        document.getElementById("printedit-format-liststyle").checked = printEdit.prefs.getBoolPref("format-liststyle");
        
        /* Hide & Undo Options */
        
        document.getElementById("printedit-hide-select").checked = printEdit.prefs.getBoolPref("hide-select");
        document.getElementById("printedit-undo-reinstate").checked = printEdit.prefs.getBoolPref("undo-reinstate");
    },
    
    onOptionsOkay: function()
    {
        /* Display Appearance Options */
        
        printEdit.prefs.setIntPref("display-mediaprint",document.getElementById("printedit-display-mediaprint").value);
        
        /* Highlight Box Options */
        
        printEdit.prefs.setCharPref("highlight-color",document.getElementById("printedit-highlight-color").color);
        printEdit.prefs.setIntPref("highlight-width",document.getElementById("printedit-highlight-width").value);
        
        /* Select Box Options */
        
        printEdit.prefs.setCharPref("select-color",document.getElementById("printedit-select-color").color);
        printEdit.prefs.setIntPref("select-width",document.getElementById("printedit-select-width").value);
        
        /* Format Properties Options */
        
        printEdit.prefs.setBoolPref("format-margin",document.getElementById("printedit-format-margin").checked);
        printEdit.prefs.setBoolPref("format-padding",document.getElementById("printedit-format-padding").checked);
        printEdit.prefs.setBoolPref("format-border",document.getElementById("printedit-format-border").checked);
        printEdit.prefs.setBoolPref("format-outline",document.getElementById("printedit-format-outline").checked);
        printEdit.prefs.setBoolPref("format-verticalalign",document.getElementById("printedit-format-verticalalign").checked);
        printEdit.prefs.setBoolPref("format-textalign",document.getElementById("printedit-format-textalign").checked);
        printEdit.prefs.setBoolPref("format-lineheight",document.getElementById("printedit-format-lineheight").checked);
        printEdit.prefs.setBoolPref("format-liststyle",document.getElementById("printedit-format-liststyle").checked);
        
        /* Hide & Undo Options */
        
        printEdit.prefs.setBoolPref("hide-select",document.getElementById("printedit-hide-select").checked);
        printEdit.prefs.setBoolPref("undo-reinstate",document.getElementById("printedit-undo-reinstate").checked);
        
        /* Hide popup */
        
        document.getElementById("printedit-options-panel").hidePopup();
    },

    onOptionsClose: function()
    {
        document.getElementById("printedit-options-panel").hidePopup();
    },

    onMenuPrintEdit: function()
    {
        printEdit.menuPrintEdit = true;
        
        if (printEdit.ffVersion == "3.6")
        {
            PrintUtils.printPreview(onEnterPrintPreview,onExitPrintPreview);
        }
        else if (printEdit.ffVersion == "4.0")
        {
            PrintUtils.printPreview(PrintPreviewListener);
        }
    },

    /********************************************************************/

    /* Button commands */

    cmdDeselect: function()
    {
        printEdit.destroyBoxes();
        printEdit.deselectElements();
        
        printEdit.disableEditButtons();
    },

    cmdFormat: function()
    {
        var selectIndex,element;
        
        for (selectIndex = 0; selectIndex < printEdit.selectCount[printEdit.cmdNumber]; selectIndex++)
        {
            element = printEdit.selectElement[printEdit.cmdNumber][selectIndex];
            if (!element.hasAttribute("printedit-format"))  /* element not already formatted */
            {
                printEdit.formatElement(element);
            }
        }
        
        printEdit.destroyBoxes();
        printEdit.nextCommand();
        
        printEdit.disableEditButtons();
        printEdit.enableUndoButtons();
    },

    cmdHide: function()
    {
        var selectIndex,element;
        
        for (selectIndex = 0; selectIndex < printEdit.selectCount[printEdit.cmdNumber]; selectIndex++)
        {
            element = printEdit.selectElement[printEdit.cmdNumber][selectIndex];
            if (printEdit.prefs.getBoolPref("hide-select"))
            {
                if (!element.hasAttribute("printedit-hide"))  /* element not already hidden */
                {
                    element.setAttribute("printedit-hide",printEdit.cmdNumber + ";opacity:" + element.style.opacity + ";");
                    element.style.setProperty("opacity","0","");
                }
            }
            else
            {
                if (!element.hasAttribute("printedit-hide"))  /* should never fail - because it was possible to select this element */
                {
                    element.setAttribute("printedit-hide",printEdit.cmdNumber + ";visibility:" + element.style.visibility + ";");
                    element.style.setProperty("visibility","hidden","");
                }
            }
        }
        
        printEdit.destroyBoxes();
        printEdit.nextCommand();
        
        printEdit.disableEditButtons();
        printEdit.enableUndoButtons();
    },

    cmdDelete: function()
    {
        var selectIndex,element;
        
        for (selectIndex = 0; selectIndex < printEdit.selectCount[printEdit.cmdNumber]; selectIndex++)
        {
            element = printEdit.selectElement[printEdit.cmdNumber][selectIndex];
            if (!element.hasAttribute("printedit-delete"))  /* should never fail - because it was possible to select this element */
            {
                element.setAttribute("printedit-delete",printEdit.cmdNumber + ";display:" + element.style.display + ";");
                element.style.setProperty("display","none","");
            }
        }

        printEdit.destroyBoxes();
        printEdit.nextCommand();
        
        printEdit.disableEditButtons();
        printEdit.enableUndoButtons();
    },

    cmdHideExcept: function()
    {
        printEdit.hideExcept(printEdit.sourceBrowser.contentDocument.body);
        
        printEdit.destroyBoxes();
        printEdit.nextCommand();
        
        printEdit.disableEditButtons();
        printEdit.enableUndoButtons();
    },

    cmdDeleteExcept: function()
    {
        printEdit.deleteExcept(printEdit.sourceBrowser.contentDocument.body);

        printEdit.destroyBoxes();
        printEdit.nextCommand();
        
        printEdit.disableEditButtons();
        printEdit.enableUndoButtons();
    },

    cmdUndo: function()
    {
        var selectIndex;
        
        printEdit.destroyBoxes();
        printEdit.deselectElements();
        printEdit.cmdNumber--;
        
        printEdit.undoCommands(printEdit.sourceBrowser.contentDocument.body);
        
        if (printEdit.prefs.getBoolPref("undo-reinstate"))
        {
            for (selectIndex = 0; selectIndex < printEdit.selectCount[printEdit.cmdNumber]; selectIndex++)
            {
                printEdit.selectBox[selectIndex] = printEdit.createBox("select");
                printEdit.showBox(printEdit.selectBox[selectIndex],printEdit.selectElement[printEdit.cmdNumber][selectIndex]);
            }
            
            printEdit.enableEditButtons();
        }
        else 
        {
            printEdit.deselectElements();
            
            printEdit.disableEditButtons();
        }

        if (printEdit.cmdNumber == 0) printEdit.disableUndoButtons();
    },

    cmdUndoAll: function()
    {
        var num;
            
        printEdit.destroyBoxes();
        printEdit.cmdNumber--;
        
        while (printEdit.cmdNumber >= 0)
        {
            printEdit.deselectElements();
            printEdit.cmdNumber--;
        }
        
        printEdit.cmdNumber = -1;
        
        printEdit.undoCommands(printEdit.sourceBrowser.contentDocument.body);

        printEdit.cmdNumber = 0;
        
        printEdit.disableEditButtons();
        printEdit.disableUndoButtons();
    },

    /********************************************************************/

    /* Debug function */

    debugMessage: function(module,information)
    {
        var info;

        printEdit.debugCount++;
        info = document.getElementById("printedit-debug-info");
        info.label = printEdit.debugCount + " - " + module + " - " + information;
    },
};

/************************************************************************/

/* Start Print Edit */

window.addEventListener("load",printEdit.onLoad,false);

printEdit.prefsObserver.register();

/************************************************************************/
